
#include<iostream>
#include<sstream>

#include "LightEntity.hh"

using namespace std;

/** Check if the LightEntity has an attribute with the given name.
 * @param attrName Name of the attribute to check
 * @return True if the entity has this attribute
 */
bool LightEntity::hasAttributeOfName(std::string const& attrName) const {
	return numberAttrs.hasAttributeOfName(attrName) || stringAttrs.hasAttributeOfName(attrName) ||
		boolAttrs.hasAttributeOfName(attrName) || entityAttrs.hasAttributeOfName(attrName);
}

/** Check if a given attribute is static or not.
 * If the attribute does not exist, gives a warning.
 * @param attrName Name of the attribute to check
 * @return True if the attribute is static. False either if the attribute is not static or does not exist.
 */
bool LightEntity::isStatic(string const& attrName) const {
	if( !(this->hasAttributeOfName(attrName)) ) {
		cerr << "WARNING: trying to make test isStatic on unknown attribute \"" << attrName;
		cerr << "\", false returned" << endl;
		return false;
	}
	return numberAttrs.isStatic(attrName) || stringAttrs.isStatic(attrName) ||
		boolAttrs.isStatic(attrName) || entityAttrs.isStatic(attrName);
}

/** Check if a given attribute is dynamic or not.
 * If the attribute does not exist, gives a warning.
 * @param attrName Name of the attribute to check
 * @return True if the attribute is dynamic. False either if the attribute is not dynamic or does not exist.
 */
bool LightEntity::isDynamic(string const& attrName) const {
	if( !(this->hasAttributeOfName(attrName)) ) {
		cerr << "WARNING: trying to make test isDynamic on unknown attribute \"" << attrName;
		cerr << "\", false returned" << endl;
		return false;
	}
	return numberAttrs.isDynamic(attrName) || stringAttrs.isDynamic(attrName) ||
		boolAttrs.isDynamic(attrName) || entityAttrs.isDynamic(attrName);
}

/** Check if a given attribute is an atom attribute or not.
 * If the attribute does not exist, gives a warning.
 * @param attrName Name of the attribute to check
 * @return True if the attribute is an atom. False either if the attribute is not an atom or does not exist.
 */
bool LightEntity::isAtom(string const& attrName) const {
	if( !(this->hasAttributeOfName(attrName)) ) {
		cerr << "WARNING: trying to make test isAtom on unknown attribute \"" << attrName;
		cerr << "\", false returned" << endl;
		return false;
	}
	return numberAttrs.isAtom(attrName) || stringAttrs.isAtom(attrName) ||
		boolAttrs.isAtom(attrName) || entityAttrs.isAtom(attrName);
}

/** Check if a given attribute is a set attribute or not.
 * If the attribute does not exist, gives a warning.
 * @param attrName Name of the attribute to check
 * @return True if the attribute is a set. False either if the attribute is not a set or does not exist.
 */
bool LightEntity::isSet(string const& attrName) const {
	if( !(this->hasAttributeOfName(attrName)) ) {
		cerr << "WARNING: trying to make test isVector on unknown attribute \"" << attrName;
		cerr << "\", false returned" << endl;
		return false;
	}
	return numberAttrs.isSet(attrName) || stringAttrs.isSet(attrName) ||
		boolAttrs.isSet(attrName) || entityAttrs.isSet(attrName);
}

/** Get the complete list of attributes.
 * @return Return the list of all attributes
 */
list<string> LightEntity::getAllAttributeNames() const {
	list<string> attrList;
	numberAttrs.getAllAttributeNames(attrList);
	stringAttrs.getAllAttributeNames(attrList);
	boolAttrs.getAllAttributeNames(attrList);
	entityAttrs.getAllAttributeNames(attrList);
	return attrList;
}

/** Make a string from the Entity.
 * @return String representing the Entity
 */
string LightEntity::toString() const {
	ostringstream st;
	st << "----- " << name << "(type: " << type << ")" << " -----" << endl;
	if(!numberAttrs.empty()) {
		st << "** Number attributes **" << endl; numberAttrs.toStringStream(st); st << endl;
	}
	if(!stringAttrs.empty()) {
		st << "** String attributes **" << endl; stringAttrs.toStringStream(st); st << endl;
	}
	if(!boolAttrs.empty()) {
		st << "** Bool attributes **" << endl; boolAttrs.toStringStream(st); st << endl;
	}
	if(!entityAttrs.empty()) {
		st << "** Entity attributes **" << endl; entityAttrs.toStringStream(st); st << endl;
	}
	if(this->isUnknown()) st << "\t** Entity is UNKNOWN by the agent **" << endl;
	if(this->isKnown()) st << "\t** Entity is KNOWN by the agent **" << endl;
	if(!unknownAttrs.empty()) {
		st << "\t** UNKNOWN attributes **" << endl;
		for(std::set<string>::iterator it = unknownAttrs.begin(); it != unknownAttrs.end(); it++) 
			st << "\t" << *it << endl;
	}
	if(!knownAttrs.empty()) {
		st << "\t** KNOWN attributes **" << endl;
		for(std::set<string>::iterator it = knownAttrs.begin(); it != knownAttrs.end(); it++) 
			st << "\t" << *it << endl;
	}
	return st.str();
}

/** Set the unknown fact to the given value.
 * @param f Value to set unknown to
 */
void LightEntity::setUnknown(bool f) {
	this->unknown = f;
}

/** Set the unknown fact to the given value.
 * @param f Value to set unknown to
 */
void LightEntity::setKnown(bool f) {
	this->known = f;
}

/** Set an attribute as unknown depending of the given value.
 * @param attrName Attribute to set
 * @param f If the attribute is already unknown and f is false, removes the attribute. Otherwise if the attribute is not in the unknown list and f is true thenn adds it to the unknown list.
 */
void LightEntity::setAttrUnknown(const std::string& attrName, bool f) {
	if( !(this->hasAttributeOfName(attrName)) ) 
		cerr << "WARNING: trying to set unknown on non existing attribute \"" << attrName << endl;
	else {
		std::set<string>::iterator it = unknownAttrs.find(attrName);
		if(it != unknownAttrs.end()) {
			if(!f) unknownAttrs.erase(it);
		}
		else {
			if(f) unknownAttrs.insert(attrName);
		}
	}
}

/** Set an attribute as known depending of the given value.
 * @param attrName Attribute to set
 * @param f If the attribute is already known and f is false, removes the attribute. Otherwise if the attribute is not in the known list and f is true thenn adds it to the unknown list.
 */
void LightEntity::setAttrKnown(const std::string& attrName, bool f) {
	if( !(this->hasAttributeOfName(attrName)) ) 
		cerr << "WARNING: trying to set known on non existing attribute \"" << attrName << endl;
	else {
		std::set<string>::iterator it = knownAttrs.find(attrName);
		if(it != knownAttrs.end()) {
			if(!f) knownAttrs.erase(it);
		}
		else {
			if(f) knownAttrs.insert(attrName);
		}
	}
}

/** Retrun the value of the given attribute as a string.
 * (Useful for connection with IxTeT (old) now for TimeProjection.)
 * @param attrName Attribute value to get
 * @return Value as a string
 */
string LightEntity::operator[](string const& attrName) const
{
	ostringstream st;
	if(numberAttrs.hasAtomOfName(attrName)) st << numberAttrs.getAtom(attrName);
	else if(stringAttrs.hasAtomOfName(attrName)) st << stringAttrs.getAtom(attrName);
	else if(boolAttrs.hasAtomOfName(attrName)) st << boolAttrs.getAtom(attrName);
	else if(entityAttrs.hasAtomOfName(attrName)) {
		if(entityAttrs.getAtom(attrName)) st << entityAttrs.getAtom(attrName)->getName();
		else st << "NULL";
	}
	else {
		cerr << "ERROR: trying to access to an atom attribute \"" << attrName;
		cerr << "\" with operator [], no such attribute found" << endl;
		cerr << "empty string returned instead." << endl;
	}
	return st.str();
}

/** Retrun the value of the given attribute as a set of strings.
 * (Useful for connection with IxTeT (old) now for TimeProjection.)
 * @param attrName Attribute value to get
 * @return Value as a set of strings
 */
set<string> LightEntity::operator()(string const& attrName) const
{
	set<string> setExit;
	ostringstream st;
	if(numberAttrs.hasSetOfName(attrName)) {
		set<number> SetN = numberAttrs.getSet(attrName);
		for(set<number>::const_iterator itN = SetN.begin() ; itN != SetN.end(); ++itN) {
			st.str(""); st << *itN;
			setExit.insert(st.str());
		}
	}
	else if(stringAttrs.hasSetOfName(attrName)) {
		set<string> SetS = stringAttrs.getSet(attrName);
		for(set<string>::const_iterator itS = SetS.begin() ; itS != SetS.end(); ++itS)
			setExit.insert(*itS);
	}
	else if(boolAttrs.hasSetOfName(attrName)) {
		set<bool> SetB = boolAttrs.getSet(attrName);
		for(set<bool>::const_iterator itB = SetB.begin() ; itB != SetB.end(); ++itB) {
			st.str(""); st << *itB;
			setExit.insert(st.str());
		}
	}
	else if(entityAttrs.hasSetOfName(attrName)) {
		set<LightEntity*> SetE = entityAttrs.getSet(attrName);
		for(set<LightEntity*>::const_iterator itE = SetE.begin() ; itE != SetE.end(); ++itE)
			setExit.insert((*itE)->getName());
	}
	return setExit;
}
