/*
 * Copyright (C) 2007 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#if !defined HATP_SOCIAL_RULE_BASE_HH_
#define HATP_SOCIAL_RULE_BASE_HH_

#include<string>
#include<vector>
#include<list>
#include<map>

#include "TaskList.hh"

class LightWorldBase;
class TimeProjection;
class PlanTree;

typedef std::pair<std::vector<TaskList*>, std::pair<std::map<TaskListNode*, std::pair<TaskList*, std::string> >, std::vector<std::string> > > badDecompoDescription;

typedef float (*undesirableStateFunc)(LightWorldBase const&, TimeProjection const&);
typedef std::pair<TaskList*, std::vector<std::string> > (*undesirableSequenceInitFunc)();
typedef float (*undesirableSequenceFunc)(TaskList const&, std::vector<std::string> const&, LightWorldBase const&, TaskList const&, TimeProjection const&);
typedef float (*effortBalancingFunc)(LightWorldBase const&, std::map<std::string, float> const& efforts);
typedef std::set<std::string> (*intricacyControlInitFunc)();
typedef float (*intricacyControlFunc)(TaskList const&, std::set<std::string> const&);
typedef std::set<std::string> (*wastedTimeInitFunc)();
typedef float (*wastedTimeFunc)(TaskList const&, TimeProjection const&, std::set<std::string> const&);
typedef float (*abstractionLegibilityFunc)(PlanTree const&, PlanTree const&);
typedef badDecompoDescription (*badDecompositionInitFunc)();
typedef float (*badDecompositionFunc)(std::vector<TaskList*> const&, std::map<TaskListNode*, std::pair<TaskList*, std::string> > const&, std::vector<std::string> const&, LightWorldBase const&, PlanTree const&);

class SocialRuleBase
{
private:
	/**
	* the list of undesirable state functions
	*/
	std::map<std::string,undesirableStateFunc> undesirableStates;

	/**
	* the list of undesirable sequence functions
	*/
	std::map<std::string, std::pair<undesirableSequenceInitFunc, undesirableSequenceFunc> > undesirableSequencesFunctions;

	/**
	* the resulting data of undesirable sequence init functions
	*/
	std::map<std::string, std::pair<TaskList*, std::vector<std::string> > > undesirableSequencesData;

	/**
	* the function defining the effort balancing rule
	*/
	effortBalancingFunc effortBalancingRule;

	/**
	* the functions defining the intricacy control rule
	*/
	std::pair<intricacyControlInitFunc, intricacyControlFunc> intricacyControlFunctions;

	/**
	* the resulting data of intricacy control init function
	*/
	std::set<std::string> intricacyControlData;

	/**
	* the functions defining the wasted time rule
	*/
	std::pair<wastedTimeInitFunc, wastedTimeFunc> wastedTimeFunctions;

	/**
	* the resulting data of wasted time init function
	*/
	std::set<std::string> wastedTimeData;

	/**
	* the function defining the abstraction legibility
	*/
	abstractionLegibilityFunc abstractionLegibilityRule;

	/**
	* the list of bad decomposition functions
	*/
	std::map<std::string, std::pair<badDecompositionInitFunc, badDecompositionFunc> > badDecompositionsFunctions;

	/**
	* the resulting data of bad decompo init functions
	*/
	std::map<std::string, badDecompoDescription> badDecompositionsData;

public:
	explicit SocialRuleBase();
	explicit SocialRuleBase(SocialRuleBase const& srb);
	std::map<std::string,undesirableStateFunc> const& getUndesirableStates() const;
	std::map<std::string, std::pair<undesirableSequenceInitFunc, undesirableSequenceFunc> > const& getUndesirableSequencesFunctions() const;
	std::map<std::string, std::pair<TaskList*, std::vector<std::string> > > const& getUndesirableSequencesData() const;
	effortBalancingFunc getEffortBalancingRule() const;
	std::pair<intricacyControlInitFunc, intricacyControlFunc> getIntricacyControlFunctions() const;
	std::pair<wastedTimeInitFunc, wastedTimeFunc> getWastedTimeFunctions() const;
	abstractionLegibilityFunc getAbstractionLegibilityRule() const;
	std::map<std::string, std::pair<badDecompositionInitFunc, badDecompositionFunc> > const& getBadDecompositionsFunctions() const;
	std::map<std::string, badDecompoDescription> getBadDecompositionsData() const;
	void add(std::string const& name, undesirableStateFunc usf);
	void add(std::string const& name, undesirableSequenceInitFunc usfInit, undesirableSequenceFunc usf);
	void add(effortBalancingFunc ebf);
	void add(intricacyControlInitFunc icfInit, intricacyControlFunc icf);
	void add(wastedTimeInitFunc wtfInit, wastedTimeFunc wtf);
	void add(abstractionLegibilityFunc alf);
	void add(std::string const& name, badDecompositionInitFunc bdfInit, badDecompositionFunc bdf);
	bool effortBalancingRuleDefined() const;
	bool intricacyControlRuleDefined() const;
	bool wastedTimeRuleDefined() const;
	bool abstractionLegibilityRuleDefined() const;
	badDecompositionFunc getBadDecompositionFunction(std::string const& name) const;
	undesirableSequenceFunc getUndesirableSequenceFunction(std::string const& name) const;
	std::pair<TaskList*, std::vector<std::string> > getUndesirableSequenceData(std::string const& name) const;
	badDecompoDescription getBadDecompositionData(std::string const& name) const;
	std::set<std::string> const& getIntricacyControlData() const;
	std::set<std::string> const& getWastedTimeData() const;
	void init();
	SocialRuleBase& operator=(SocialRuleBase const& other);
	~SocialRuleBase();
};

/**
 * constructor: empty
 */
inline SocialRuleBase::SocialRuleBase() : effortBalancingRule(NULL), abstractionLegibilityRule(NULL)
{
	intricacyControlFunctions = std::make_pair((intricacyControlInitFunc)NULL, (intricacyControlFunc)NULL);
	wastedTimeFunctions = std::make_pair((wastedTimeInitFunc)NULL, (wastedTimeFunc)NULL);
}

/**
 * return undesirableStates
 */
inline std::map<std::string,undesirableStateFunc> const& SocialRuleBase::getUndesirableStates() const
{
	return undesirableStates;
}

/**
 * return undesirableSequencesFunctions
 */
inline std::map<std::string, std::pair<undesirableSequenceInitFunc, undesirableSequenceFunc> > const& SocialRuleBase::getUndesirableSequencesFunctions() const
{
	return undesirableSequencesFunctions;
}

/**
 * return undesirableSequencesData
 */
inline std::map<std::string, std::pair<TaskList*, std::vector<std::string> > > const& SocialRuleBase::getUndesirableSequencesData() const
{
	return undesirableSequencesData;
}

/**
 * return effortBalancingRule
 */
inline effortBalancingFunc SocialRuleBase::getEffortBalancingRule() const
{
	return effortBalancingRule;
}

/**
 * return intricacyControlFunctions
 */
inline std::pair<intricacyControlInitFunc, intricacyControlFunc> SocialRuleBase::getIntricacyControlFunctions() const
{
	return intricacyControlFunctions;
}

/**
 * return wastedTimeFunctions
 */
inline std::pair<wastedTimeInitFunc, wastedTimeFunc> SocialRuleBase::getWastedTimeFunctions() const
{
	return wastedTimeFunctions;
}

/**
 * return intricacyControlRule
 */
inline abstractionLegibilityFunc SocialRuleBase::getAbstractionLegibilityRule() const
{
	return abstractionLegibilityRule;
}

/**
 * return badDecompositionsFunctions
 */
inline std::map<std::string, std::pair<badDecompositionInitFunc, badDecompositionFunc> > const& SocialRuleBase::getBadDecompositionsFunctions() const
{
	return badDecompositionsFunctions;
}

/**
 * return badDecompositionsData
 */
inline std::map<std::string, badDecompoDescription> SocialRuleBase::getBadDecompositionsData() const
{
	return badDecompositionsData;
}

/**
 * set effortBalancingRule to ebf
 */
inline void SocialRuleBase::add(effortBalancingFunc ebf)
{
	effortBalancingRule = ebf;
}

/**
 * set intricacyControlFunctions to the pair (icfInit, icf)
 */
inline void SocialRuleBase::add(intricacyControlInitFunc icfInit, intricacyControlFunc icf)
{
	intricacyControlFunctions = std::make_pair(icfInit, icf);
}

/**
 * set intricacyControlFunctions to the pair (wtfInit, wtf)
 */
inline void SocialRuleBase::add(wastedTimeInitFunc wtfInit, wastedTimeFunc wtf)
{
	wastedTimeFunctions = std::make_pair(wtfInit, wtf);
}

/**
 * set abstractionLegibilityRule to alf
 */
inline void SocialRuleBase::add(abstractionLegibilityFunc alf)
{
	abstractionLegibilityRule = alf;
}

/**
 * return effortBalancingRule!=NULL
 */
inline bool SocialRuleBase::effortBalancingRuleDefined() const
{
	return effortBalancingRule != NULL;
}

/**
 * return intricacyControlFunctions.first != NULL && intricacyControlFunctions.second != NULL
 */
inline bool SocialRuleBase::intricacyControlRuleDefined() const
{
	return intricacyControlFunctions.first != NULL && intricacyControlFunctions.second != NULL;
}

/**
 * return wastedTimeFunctions.first != NULL && wastedTimeFunctions.second != NULL
 */
inline bool SocialRuleBase::wastedTimeRuleDefined() const
{
	return wastedTimeFunctions.first != NULL && wastedTimeFunctions.second != NULL;
}

/**
 * return abstractionLegibilityRule!=NULL
 */
inline bool SocialRuleBase::abstractionLegibilityRuleDefined() const
{
	return abstractionLegibilityRule != NULL;
}

/**
 * return function associated to name in badDecompositions, return NULL if
 * name is not in badDecompositions
 */
inline badDecompositionFunc SocialRuleBase::getBadDecompositionFunction(std::string const& name) const
{
	std::map<std::string, std::pair<badDecompositionInitFunc, badDecompositionFunc> >::const_iterator it = badDecompositionsFunctions.find(name);
	if(it != badDecompositionsFunctions.end())
		return (it->second).second;

	return NULL;
}

/**
 * return function associated to name in undesirableSequences, return NULL if
 * name is not in undesirableSequences
 */
inline undesirableSequenceFunc SocialRuleBase::getUndesirableSequenceFunction(std::string const& name) const
{
	std::map<std::string, std::pair<undesirableSequenceInitFunc, undesirableSequenceFunc> >::const_iterator it = undesirableSequencesFunctions.find(name);
	if(it != undesirableSequencesFunctions.end())
		return (it->second).second;

	return NULL;
}

/**
 * return data associated to name in undesirableSequencesData, return an empty pair if
 * name is not in undesirableSequences
 */
inline std::pair<TaskList*, std::vector<std::string> > SocialRuleBase::getUndesirableSequenceData(std::string const& name) const
{
	std::map<std::string, std::pair<TaskList*, std::vector<std::string> > >::const_iterator it = undesirableSequencesData.find(name);
	if(it != undesirableSequencesData.end())
		return it->second;

	return std::make_pair((TaskList*)NULL, std::vector<std::string>(0));
}

/**
 * return data associated to name in badDecompositionsData, return an empty description if
 * name is not in badDecompositionsData
 */
inline badDecompoDescription SocialRuleBase::getBadDecompositionData(std::string const& name) const
{
	std::map<std::string, badDecompoDescription>::const_iterator it = badDecompositionsData.find(name);
	if(it != badDecompositionsData.end())
		return it->second;

	return std::make_pair(std::vector<TaskList*>(0), std::make_pair(std::map<TaskListNode*, std::pair<TaskList*, std::string> >(), std::vector<std::string>(0)));
}

/**
 * return intricacyControlData
 */
inline std::set<std::string> const& SocialRuleBase::getIntricacyControlData() const
{
	return intricacyControlData;
}

/**
 * return wastedTimeData
 */
inline std::set<std::string> const& SocialRuleBase::getWastedTimeData() const
{
	return wastedTimeData;
}

/**
 * destructor: delete members of undesirableSequencesData and badDecompositionsData
 */
inline SocialRuleBase::~SocialRuleBase()
{
	std::map<std::string, std::pair<TaskList*, std::vector<std::string> > >::iterator it;
	for(it = undesirableSequencesData.begin() ; it != undesirableSequencesData.end() ; ++it)
		delete (it->second).first;

	std::map<std::string, badDecompoDescription>::iterator it2;
	unsigned int i;
	std::vector<TaskList*> vecTL;
	for(it2 = badDecompositionsData.begin() ; it2 != badDecompositionsData.end() ; ++it2)
	{
		vecTL = (it2->second).first;
		for(i=0 ; i<vecTL.size() ; ++i)
			delete vecTL[i];
	}
}

#endif
