#include<iostream>

#include "WorldBase.hh"
#include "tools.hh"

using namespace std;

/** Add an Entity to the database.
 * If there already is an Entity with the same name it gives a warning and replace the old Entity (whitout freeing the last).
 * @param e Entity to add to the database
 */
void WorldBase::add(Entity const& e) {
	if(database.find(e.getName()) == database.end()) database[e.getName()] = new Entity(e);
	else cerr << "WARNING: trying to add entity \"" << e.getName() << "\" whereas it already exists" << endl;
}

/** Clear all dynamic vectors of all the entities in the database.
 * It means that all dynamic attributes (of every entity) are set to their default value. (See AttributeSet::delallDynamicVector())
 */
void WorldBase::clearAllDynamicVectors() {
	for(map<string, Entity*>::iterator it=database.begin() ; it != database.end() ; ++it)
		(it->second)->delallDynamicVector();
}


/** Check if an Entity exists in the agent's database.
 * @param sName Name of the entity to check
 * @return True if the entity exists
 */
bool WorldBase::containEntity(std::string const& sName) const {
	return database.find(sName) != database.end();
}

/** Check if an Entity (with a specific type) exists in the agent's database.
 * @param sName Name of the entity to check
 * @param sType The type the entity need to be
 * @return True if the entity exists and is of the given type
 */
bool WorldBase::containEntity(std::string const& sName, std::string const& sType) const {
	map<string, Entity*>::const_iterator it = database.find(sName);
	if(it != database.end() && it->second->getType() == sType)
		return true;

	return false;
}

/** Find an Entity in the world database using its name and type.
 * @param sName The entity to find
 * @param sType The type of the entity
 * @return A pointer to the entity or NULL if the entity does not exist or is not of the given type
 */
Entity* WorldBase::find(std::string const& sName, std::string const& sType) {
	map<string, Entity*>::iterator it = database.find(sName);
	if(it != database.end() && it->second->getType() == sType)
		return it->second;
	cerr << "ERROR when trying to access to an unknown Entity \"" << sName << "\" of type \"" << sType;
	cerr << "\" in the WorldBase." << endl << "NULL returned instead" << endl;
	return NULL;
}

/** Find an entity in the world database using its name and type.
 * @param sName The entity to find
 * @param sType The type of the entity
 * @return A const pointer to the entity or NULL if the entity does not exist or is not of the given type
 */
Entity const* WorldBase::find(std::string const& sName, std::string const& sType) const {
	map<string, Entity*>::const_iterator it = database.find(sName);
	if(it != database.end() && it->second->getType() == sType)
		return it->second;
	cerr << "ERROR when trying to access to an unknown Entity \"" << sName << "\" of type \"" << sType;
	cerr << "\" in the WorldBase." << endl << "NULL returned instead" << endl;
	return NULL;
}

/** Find a set of all entities matching the given type.
 * @param sType The type the entities need to match
 * @return Set of entities matching the given type
 */
vector<Entity*> WorldBase::findByType(string const& sType) const {
	vector<Entity*> vec;
	for(map<string,Entity*>::const_iterator it=database.begin() ; it!=database.end() ; ++it)
		if(it->second->getType() == sType)
			vec.push_back(it->second);
	return vec;
}

/** Find a set of all entities that have a given attribute.
 * @param attrName The attribute the entities need to have
 * @return Set of entities having the given attribute
 */
vector<Entity*> WorldBase::findByAttribute(string const& attrName) const {
	vector<Entity*> vec;
	for(map<string,Entity*>::const_iterator it=database.begin() ; it!=database.end() ; ++it) {
		if(it->second->hasAttributeOfName(attrName)) {
			vec.push_back(it->second);
		}
	}
	return vec;
}

/** Set the name of this database.
 * @param agentName New name fot this database
 */
void WorldBase::setMyselfName(std::string agentName) {
	this->myselfName = agentName;
	this->activeBM = true;
}

/** Create a WorldBaseAgent for a given agent.
 * If the agent already had a database it gives a warning and does not replace the previous.
 * @param agentName The name of the agent whose base has to be initialised
 */
void WorldBase::createWBForAgent(string const& agentName) {
	if(this->WBAgents.find(agentName) == this->WBAgents.end()) 
		this->WBAgents[agentName] = new WorldBaseAgent();
	else cerr << "WARNING: trying to add Agent \"" << agentName << "\" DB whereas it already exists" << endl;
}

/** Add an entity to a given agent's database.
 * @param agentName Agent to add the entity to
 * @param e Entity to add
 */
void WorldBase::addForAgent(string const& agentName, Entity const& e) {
	map<string, WorldBaseAgent*>::iterator it = WBAgents.find(agentName);
	if(it != WBAgents.end()) it->second->add(e);
	else cerr << "WARNING: trying to add \"" << e.getName() << "\" in an unexisting agent database" << endl;
}

/** Try to add an entity to an agent's database.
 * If the agent does not already have a database, it is created. If the entity already exists in the agent's database then a pointer to this entiy is returned. If the entity does not exist, it is copied from this database (if possible).
 * @param agentName Name of the agent to add the entity to
 * @param entityName Name of the entity to add
 * @return Pointer to this entity or NULL in case of error (the agent or the entity does not exist)
 */
Entity* WorldBase::initForAgent(std::string agentName, std::string entityName) {
	if(this->WBAgents.find(agentName) == this->WBAgents.end()) {
		this->createWBForAgent(agentName);
	}
	map<string, WorldBaseAgent*>::iterator it = WBAgents.find(agentName);
	if(it != WBAgents.end()) {
		Entity* tmp = it->second->find(entityName);
		if(tmp != NULL) {
			return tmp;
		}
		else {
			tmp = this->find(entityName);
			if(tmp == NULL) return NULL;
			Entity* newEnt = new Entity(*tmp);
			newEnt->initAttrsKnownUnknown();
			newEnt->setKnown(false);
			newEnt->setUnknown(false);
			it->second->add(*newEnt);
			return it->second->find(entityName);
		}
	}
	return NULL;
}

/** Update an attribute for an agent's entity using the value from another agent.
 * @param a1 Agent from which to copy the value
 * @param a2 Agent that has the entity to be updated
 * @param e Name of the entity to update
 * @param attr Name of the entity's attribute to update
 */
void WorldBase::updateForAgent(std::string a1, std::string a2, std::string e, std::string attr) {
	Entity* efa1 = this->findForAgent(a1, e);
	Entity* efa2 = this->findForAgent(a2, e);
	if(attr == string("NULL")) {
		if(efa2!=NULL) delete(efa2);
		efa2 = new Entity(*efa1);
	}
	else {
		if(efa1 != NULL && efa2 != NULL) {
			if(efa1->isAtom(attr)) {
				if(efa1->getNumberAttrs().hasAttributeOfName(attr)) efa2->get<number>(attr) = efa1->get<number>(attr);
				if(efa1->getStringAttrs().hasAttributeOfName(attr)) efa2->get<string>(attr) = efa1->get<string>(attr);
				if(efa1->getBoolAttrs().hasAttributeOfName(attr)) efa2->get<bool>(attr) = efa1->get<bool>(attr);
				if(efa1->getEntityAttrs().hasAttributeOfName(attr)) efa2->get<Entity*>(attr) = efa1->get<Entity*>(attr);
			}
			else {
				if(efa1->getNumberAttrs().hasAttributeOfName(attr)) efa2->get<set<number> >(attr) = efa1->get<set<number> >(attr);
				if(efa1->getStringAttrs().hasAttributeOfName(attr)) efa2->get<set<string> >(attr) = efa1->get<set<string> >(attr);
				if(efa1->getBoolAttrs().hasAttributeOfName(attr)) efa2->get<set<bool> >(attr) = efa1->get<set<bool> >(attr);
				if(efa1->getEntityAttrs().hasAttributeOfName(attr)) efa2->get<set<Entity*> >(attr) = efa1->get<set<Entity*> >(attr);
			}
			efa2->setAttrKnown(attr, false);
			efa2->setAttrUnknown(attr, false);
		}
		else {
			cerr << "ERROR in UpdateForAgent. Something is missing in DB." << endl;
		}
	}
}

/** Create a string representing the WorldBase.
 * First put the list of entities in this database, then put every agent database. (See WorldBaseAgent::toString().)
 * @return A string representing the world entity databse
 */
string WorldBase::toString() const {
	string s = "***** Start Myself WorldBase Printing *****\n";
	for(map<string,Entity*>::const_iterator it=database.begin() ; it!=database.end() ; ++it)
		s += it->second->toString() + "\n";
	s += "***** End Myself WorldBase Printing *****\n";
	for(map<string,WorldBaseAgent*>::const_iterator it2=WBAgents.begin() ; it2!=WBAgents.end() ; ++it2) {
		s += "\nFOR AGENT "+it2->first+" :\n\n";
		s += it2->second->toString() + "\n";
	}
	return s;
}

/** Return a reference to an Entity from the agent's database.
 * If the entity does not exist, an empty entity is return instead and a warning is generated.
 * @param c Entity to return
 * @return Reference to the entity, or to an empty entity
 */
Entity& WorldBase::operator[](const char* c) {
	std::map<std::string, Entity*>::iterator it = database.find(c);
	if(it != database.end()) return *(it->second);
	else {
		cerr << "ERROR when trying to access to an Entity in the WorldBase: " << c;
		cerr << " has not been found" << endl << "default Entity returned instead" << endl;
		return defaultEntity;
	}
}

/** Return a reference to an Entity from the agent's database.
 * If the entity does not exist, an empty entity is return instead and a warning is generated.
 * @param s Entity to return
 * @return Reference to the entity, or to an empty entity
 */
Entity& WorldBase::operator[](std::string const& s) {
	std::map<std::string, Entity*>::iterator it = database.find(s);
	if(it != database.end()) return *(it->second);
	else {
		cerr << "ERROR when trying to access to an Entity in the WorldBase: " << s;
		cerr << " has not been found" << endl << "default Entity returned instead" << endl;
		return defaultEntity;
	}
}

/** Return a reference to an Entity from the agent's database.
 * If the entity does not exist, an empty entity is return instead and a warning is generated.
 * @param c Entity to return
 * @return Const reference to the entity, or to an empty entity
 */
Entity const& WorldBase::operator[](const char* c) const {
	std::map<std::string, Entity*>::const_iterator it = database.find(c);
	if(it != database.end()) return *(it->second);
	else {
		cerr << "ERROR when trying to access to an Entity in the WorldBase: " << c;
		cerr << " has not been found" << endl << "default Entity returned instead" << endl;
		return defaultEntity;
	}
}

/** Return a reference to an Entity from the agent's database.
 * If the entity does not exist, an empty entity is return instead and a warning is generated.
 * @param s Entity to return
 * @return Const reference to the entity, or to an empty entity
 */
Entity const& WorldBase::operator[](std::string const& s) const {
	std::map<std::string, Entity*>::const_iterator it = database.find(s);
	if(it != database.end()) return *(it->second);
	else {
		cerr << "ERROR when trying to access to an Entity in the WorldBase: " << s;
		cerr << " has not been found" << endl << "default Entity returned instead" << endl;
		return defaultEntity;
	}
}

/** Destructor : delete all Entities in database
 */
WorldBase::~WorldBase() {
	for(map<string,WorldBaseAgent*>::iterator it=WBAgents.begin() ; it!=WBAgents.end() ; ++it)
		delete it->second;
	for(map<string,Entity*>::iterator it=database.begin() ; it!=database.end() ; ++it)
		delete it->second;
}
