# Find the Antlr module
# Once found the following variables are set :
#	Antlr_FOUND	True to specify that the module has been found
#
#	Antlr_INCLUDE_DIRS	Path(s) to the headers
#
#	Antlr_LIBRARIES	Link these to use Antlr

# To help CMake in finding the Antlr module
# one can set the following variables :
# Antlr_DIR	Those variables can be either set using 'set' command of
# or			CMake, or using '-D<var>=value' while calling CMake or 
# Antlr_ROOT	finally they can be set as environment variables.
# or			Those three variables are aliases, so just set one is enough.
# AntlrROOT
#
# You can set any of them using one of the following methods :
#	1) Set it as environnement varialbe (e.g. export)
#	2) Give it as argument for cmake : cmake -DAntlr_DIR="...."
#	3) In a CMakeLists.txt, set a variable : set(Antlr_DIR ...)	

#Varaibles to change if the module changes
set(HEADER_FILES ANTLRException.hpp CharStreamException.hpp MismatchedTokenException.hpp TokenStreamException.hpp ANTLRUtil.hpp CharStreamIOException.hpp NoViableAltException.hpp TokenStreamHiddenTokenFilter.hpp ASTArray.hpp CircularQueue.hpp NoViableAltForCharException.hpp TokenStream.hpp ASTFactory.hpp CommonAST.hpp Parser.hpp TokenStreamIOException.hpp AST.hpp CommonASTWithHiddenTokens.hpp ParserSharedInputState.hpp TokenStreamRecognitionException.hpp ASTNULLType.hpp CommonHiddenStreamToken.hpp RecognitionException.hpp TokenStreamRetryException.hpp ASTPair.hpp CommonToken.hpp RefCount.hpp TokenStreamRewriteEngine.hpp ASTRefCount.hpp config.hpp SemanticException.hpp TokenStreamSelector.hpp BaseAST.hpp InputBuffer.hpp String.hpp TokenWithIndex.hpp BitSet.hpp IOException.hpp TokenBuffer.hpp TreeParser.hpp CharBuffer.hpp LexerSharedInputState.hpp Token.hpp TreeParserSharedInputState.hpp CharInputBuffer.hpp LLkParser.hpp TokenRefCount.hpp CharScanner.hpp MismatchedCharException.hpp TokenStreamBasicFilter.hpp)
set(INCLUDE_PATH_SUFFIXES include)
set(LIBRARIES libantlr.a)
set(EXECUTABLE runantlr)

foreach(header ${HEADER_FILES})
	find_path(${header}_PATH antlr/${header} PATH_SUFFIXES ${INCLUDE_PATH_SUFFIXES}
	HINTS ${Antlr_DIR} ${Antlr_ROOT} ${AntlrROOT}
	$ENV{Antlr_DIR} $ENV{Antlr_ROOT} $ENV{AntlrROOT})
	if(${header}_PATH)
		list(APPEND Antlr_INCLUDE_DIRS ${${header}_PATH})
	else(${header}_PATH)
		list(APPEND MISSING_HEADERS ${header})
	endif(${header}_PATH)
endforeach()

if(Antlr_INCLUDE_DIRS)
	list(REMOVE_DUPLICATES Antlr_INCLUDE_DIRS)
endif(Antlr_INCLUDE_DIRS)

if(MISSING_HEADERS)
	unset(Antlr_INCLUDE_DIRS)
endif(MISSING_HEADERS)

find_file(Antlr_LIBRARIES ${LIBRARIES} PATH_SUFFIXES lib 
	HINTS ${Antlr_DIR} ${Antlr_ROOT} ${AntlrROOT}
	$ENV{Antlr_DIR} $ENV{Antlr_ROOT} $ENV{AntlrROOT})

find_file(Antlr_EXECUTABLE ${EXECUTABLE} PATH_SUFFIXES bin 
	HINTS ${Antlr_DIR} ${Antlr_ROOT} ${AntlrROOT}
	$ENV{Antlr_DIR} $ENV{Antlr_ROOT} $ENV{AntlrROOT})

if(Antlr_INCLUDE_DIRS AND Antlr_LIBRARIES AND Antlr_EXECUTABLE)
	set(Antlr_FOUND TRUE)
else(Antlr_INCLUDE_DIRS AND Antlr_LIBRARIES AND Antlr_EXECUTABLE)
	if(NOT Antlr_FIND_QUIETLY)
		if(NOT Antlr_INCLUDE_DIRS)
			message(STATUS "Unable to find Antlr, the following header files are missing : ${MISSING_HEADERS}")
		else(NOT Antlr_INCLUDE_DIRS)
			message(STATUS "Header files found.")
		endif(NOT Antlr_INCLUDE_DIRS)
		if(NOT Antlr_LIBRARIES)
			message(STATUS "Unable to find Antlr library files!")
		else(NOT Antlr_LIBRARIES)
			message(STATUS "Library files found.")
		endif(NOT Antlr_LIBRARIES)
		if(NOT Antlr_EXECUTABLE)
			message(STATUS "Unable to find Antlr executable!")
		else(NOT Antlr_EXECUTABLE)
			message(STATUS "Antlr executable found.")
		endif(NOT Antlr_EXECUTABLE)
	endif(NOT Antlr_FIND_QUIETLY)
endif(Antlr_INCLUDE_DIRS AND Antlr_LIBRARIES AND Antlr_EXECUTABLE)

if(Antlr_FOUND)
	if(NOT Antlr_FIND_QUIETLY)
		message(STATUS "Found components for Antlr")
		message(STATUS "Antlr_INCLUDE_DIRS = ${Antlr_INCLUDE_DIRS}")
		message(STATUS "Antlr_LIBRARIES = ${Antlr_LIBRARIES}")
		message(STATUS "Antlr_EXECUTABLE = ${Antlr_EXECUTABLE}")
	endif(NOT Antlr_FIND_QUIETLY)
else(Antlr_FOUND)
	if(Antlr_FIND_REQUIRED)
		set(Antlr_DIR "" CACHE PATH "Paths where to additionally look for
		Antlr (alias of Antlr_ROOT and AntlrROOT)")
		set(Antlr_ROOT "" CACHE PATH "Paths where to additionally look for
		Antlr (alias of Antlr_DIR and AntlrROOT)")
		set(AntlrROOT "" CACHE PATH "Paths where to additionally look for
		Antlr (alias of Antlr_DIR and Antlr_ROOT)")
		if(NOT Antlr_FIND_QUIETLY)
			message("\nCould not find Antlr!\n")
			message("You can use any of the following variables to help CMake to find the header files and libraries :")
			message("	Antlr_DIR or Antlr_ROOT or AntlrROOT")
			message("You should use one and set it to the root path of your local Antlr install directory (if you installed it in a different folder than system root).")
			message("To set one you can either :")
			message("	- use 'set(Antlr ....)' command in the CMakeLists.txt (separator ' ')")
			message("	- use '-DAntlr_DIR=....' as an option to Cmake (separator ';')")
			message("	- set one as an environment variable (e.g. export in bash, with separator ';')\n")
		endif(NOT Antlr_FIND_QUIETLY)
		message(FATAL_ERROR "Could not find Antlr!")
	endif(Antlr_FIND_REQUIRED)
endif(Antlr_FOUND)

mark_as_advanced (
	Antlr_FOUND
	Antlr_LIBRARIES
	Antlr_INCLUDE_DIRS
	Antlr_EXECUTABLE
)
